﻿' (C) Copyright 2002-2009 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted, 
' provided that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting 
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'
Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

' This line is not mandatory, but improves loading performances
<Assembly: CommandClass(GetType(CustomFilterOverrule_VB.MyCommands))> 

Namespace CustomFilterOverrule_VB

#Region "The commands"
    Public Class MyCommands

        'Stores our global overrule instance
        Private Shared mOverrule As MyDrawOverrule

        'Highlights blocks with names containing the search string entered at user prompt.
        'If null string is entered, then previous highlight is turned off.
        <Autodesk.AutoCAD.Runtime.CommandMethod("SHOWBLOCKS")> _
        Shared Sub FindText()
            'Prompt user to enter search string
            Dim opts As New PromptStringOptions(vbCrLf & "Enter block search text: ")
            opts.AllowSpaces = True
            Dim res As PromptResult = Application.DocumentManager.MdiActiveDocument.Editor.GetString(opts)

            'If user cancelled the command, then we exit
            If res.Status <> PromptStatus.OK Then Exit Sub

            'If user entered null string, then we turn off our overruling
            If res.StringResult = "" Then
                ResetBlocks()
                Exit Sub
            End If

            'Create and register our overrule if this is first time function has run
            If mOverrule Is Nothing Then
                mOverrule = New MyDrawOverrule
                Overrule.AddOverrule(RXObject.GetClass(GetType(BlockReference)), mOverrule, False)
                mOverrule.SetCustomFilter()
            End If

            'Set search text for Overrule to that entered by user
            mOverrule.SearchText = res.StringResult

            'Turn Overruling on
            Overrule.Overruling = True
            'Regen is required to update changes on screen.
            Application.DocumentManager.MdiActiveDocument.Editor.Regen()
        End Sub

        'Changes the color used to highlight blocks
        <CommandMethod("SHOWCOLOR")> _
        Shared Sub SetColor()
            'Promot user to enter color index
            Dim opts As New PromptIntegerOptions(vbCrLf & "Enter Block Finder Color Index: ")
            opts.LowerLimit = 0
            opts.UpperLimit = 127
            'Show current highlight color index as default
            If Not mOverrule Is Nothing Then
                opts.DefaultValue = mOverrule.HighlightColor
            Else
                opts.DefaultValue = 3
            End If
            Dim res As PromptIntegerResult = Application.DocumentManager.MdiActiveDocument.Editor.GetInteger(opts)

            If res.Status <> PromptStatus.OK Then Exit Sub

            If mOverrule Is Nothing Then
                mOverrule = New MyDrawOverrule
                Overrule.AddOverrule(RXObject.GetClass(GetType(BlockReference)), mOverrule, False)
                mOverrule.SetCustomFilter()
            End If

            'If requested highlight color is a new color, then we want to change it
            If mOverrule.HighlightColor <> res.Value Then
                mOverrule.HighlightColor = res.Value
                If Overrule.Overruling Then
                    'Regen is required to update changes on screen.
                    Application.DocumentManager.MdiActiveDocument.Editor.Regen()
                End If
            End If
        End Sub

        'Turns off overrule
        Private Shared Sub ResetBlocks()
            If Not mOverrule Is Nothing Then
                Overrule.Overruling = False
                Overrule.RemoveOverrule(RXObject.GetClass(GetType(BlockReference)), mOverrule)
                mOverrule = Nothing
                Application.DocumentManager.MdiActiveDocument.Editor.Regen()
            End If
        End Sub

    End Class
#End Region



#Region "The overrule"
    Public Class MyDrawOverrule
        Inherits Autodesk.AutoCAD.GraphicsInterface.DrawableOverrule

        'The text we'll search for in our block name.
        Private mTxt As String
        'Color Index of block highlight 
        Private mColor As Short = 3

        'The color we highlight blocks with
        Public Property HighlightColor() As Short
            Get
                Return mColor
            End Get
            Set(ByVal value As Short)
                If value >= 0 And value <= 127 Then
                    mColor = value
                End If
            End Set
        End Property

        'The text we'll search for in the block name
        Public Property SearchText() As String
            Get
                Return mTxt
            End Get
            Set(ByVal value As String)
                mTxt = value.ToUpper
            End Set
        End Property

        'Overrule WorldDraw so we can draw our additional graphics
        Public Overrides Function WorldDraw(ByVal drawable As Autodesk.AutoCAD.GraphicsInterface.Drawable, ByVal wd As Autodesk.AutoCAD.GraphicsInterface.WorldDraw) As Boolean
            
            'We know this overrule is only registered for BlockReferences, so cast without checking. 
            Dim br As BlockReference = drawable

            'Now we want to draw a green box around the attributes extents
            Dim ext As Extents3d = br.Bounds
            Dim maxPt As Point3d = ext.MaxPoint
            Dim minPt As Point3d = ext.MinPoint
            Dim pts As New Point3dCollection

            'These are the vertices of the highlight box
            pts.Add(New Point3d(minPt.X, minPt.Y, minPt.Z))
            pts.Add(New Point3d(minPt.X, maxPt.Y, minPt.Z))
            pts.Add(New Point3d(maxPt.X, maxPt.Y, minPt.Z))
            pts.Add(New Point3d(maxPt.X, minPt.Y, minPt.Z))
            'pts.Add(New Point3d(minPt.X, minPt.Y, minPt.Z))

            'Store current filltype and set to FillAlways
            Dim oldFillType As Autodesk.AutoCAD.GraphicsInterface.FillType = wd.SubEntityTraits.FillType
            wd.SubEntityTraits.FillType = Autodesk.AutoCAD.GraphicsInterface.FillType.FillAlways

            'Store old graphics color and set to the color we want
            Dim oldColor As Short = wd.SubEntityTraits.Color
            wd.SubEntityTraits.Color = Me.HighlightColor

            'draw the filled polygon
            wd.Geometry.Polygon(pts)

            'restore old settings
            wd.SubEntityTraits.FillType = oldFillType
            wd.SubEntityTraits.Color = oldColor

            'Let the overruled Drawable draw itself.
            Return MyBase.WorldDraw(drawable, wd)
        End Function

        'This function is called if we call SetCustomFilter on our custom overrule.
        'We add our own code to return true if the BlockReference passed in is one we want to highlight.
        Public Overrides Function IsApplicable(ByVal overruledSubject As Autodesk.AutoCAD.Runtime.RXObject) As Boolean
            'If its an BlockReference, we check if the Block Name contains our
            If TypeOf overruledSubject Is BlockReference And SearchText <> "" Then
                Dim br As BlockReference = overruledSubject
                Dim strBlockName As String = br.Name.ToUpper
                'Return true if the filter is applicable to this object
                If strBlockName.Contains(Me.SearchText) Then
                    Return True
                End If
            End If
            'Only get to here if object isn't a BlockReference or it didn't contain our search string
            Return False
        End Function

    End Class
#End Region




End Namespace