﻿' (C) Copyright 2002-2009 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted, 
' provided that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting 
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'
Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.GraphicsInterface

' This line is not mandatory, but improves loading performances
<Assembly: CommandClass(GetType(DrawingStuff_VB.MyCommands))> 

Namespace DrawingStuff_VB

#Region "Commands"
  Public Class MyCommands

    Private Shared mDrawOverrule As DrawDrawOverrule
    Private Shared mPrimitiveOverrule As PrimitiveDrawOverrule
    Private Shared mWorldDrawDrawOverrule As WorldDrawDrawOverrule
    Private Shared index As Integer


    <CommandMethod("TOGGLEDRAWSTYLES")> _
    Public Shared Sub TogglesDrawStyles()

      Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

      ' Remove all our old overrules
      If Not mDrawOverrule Is Nothing Then
        Overrule.RemoveOverrule(RXClass.GetClass(GetType(Line)), mDrawOverrule)
        mDrawOverrule.Dispose()
        mDrawOverrule = Nothing
      End If
      If Not mWorldDrawDrawOverrule Is Nothing Then
        Overrule.RemoveOverrule(RXClass.GetClass(GetType(Line)), mWorldDrawDrawOverrule)
        mWorldDrawDrawOverrule.Dispose()
        mWorldDrawDrawOverrule = Nothing
      End If
      If Not mPrimitiveOverrule Is Nothing Then
        Overrule.RemoveOverrule(RXClass.GetClass(GetType(Line)), mPrimitiveOverrule)
        mPrimitiveOverrule.Dispose()
        mPrimitiveOverrule = Nothing
      End If

      ' Create and register the overrule corresponding to index value
      Select Case (index)
        Case 0
          ed.WriteMessage(vbLf & "*** Now drawing using temporary entities (Draw) ***" & vbLf)
          mDrawOverrule = New DrawDrawOverrule
          Overrule.AddOverrule(RXClass.GetClass(GetType(Line)), mDrawOverrule, False)

        Case 1
          ed.WriteMessage(vbLf & "*** Now drawing using graphics primitives ***" & vbLf)
          mPrimitiveOverrule = New PrimitiveDrawOverrule
          Overrule.AddOverrule(RXClass.GetClass(GetType(Line)), mPrimitiveOverrule, False)

        Case 2
          ed.WriteMessage(vbLf & "*** Now drawing using temporary entities (WorldDraw) ***" & vbLf)
          mWorldDrawDrawOverrule = New WorldDrawDrawOverrule
          Overrule.AddOverrule(RXClass.GetClass(GetType(Line)), mWorldDrawDrawOverrule, False)

      End Select

      'Increment index so we use next overrule type when this function is next called
      index = index + 1
      If index = 3 Then index = 0

      'Regen graphics to show changes
      ed.Regen()

    End Sub


    <CommandMethod("TOGGLEOVERRULE")> _
    Public Shared Sub ToggleOverrule()
      '
      'Toggle value of Overruling, then regen graphics to show changes.
      Overrule.Overruling = Not Overrule.Overruling
      Application.DocumentManager.MdiActiveDocument.Editor.Regen()

    End Sub

  End Class
#End Region



#Region "Overrule using graphics primitives"
  Public Class PrimitiveDrawOverrule
    Inherits DrawableOverrule

    'Draw using graphics primitives
    Public Overrides Function WorldDraw(ByVal drawable As Autodesk.AutoCAD.GraphicsInterface.Drawable, ByVal wd As Autodesk.AutoCAD.GraphicsInterface.WorldDraw) As Boolean
      Dim myLine As Line = drawable
      'Draw a circle
      wd.Geometry.Circle(myLine.StartPoint + 0.5 * myLine.Delta, myLine.Length / 4, myLine.Normal)
      'Draw some text using the default textstyle
      Dim txtStyle As New TextStyle
      wd.Geometry.Text(myLine.StartPoint, myLine.Normal, myLine.Delta, "Graphics primitives", True, txtStyle)
      'Draw an arc
      wd.Geometry.CircularArc(myLine.EndPoint, myLine.Length / 4, myLine.Normal, myLine.Normal.CrossProduct(myLine.Delta.GetNormal), Math.PI, ArcType.ArcSimple)
      Return MyBase.WorldDraw(drawable, wd)
    End Function

  End Class
#End Region


#Region "Overrule using Draw method"
  Public Class DrawDrawOverrule
    Inherits DrawableOverrule

    Dim mText As DBText
    Dim mCircle As Circle
    Dim mArc As Arc

    'Create our temporary entities once and keep them for the lifetime of the overrule
    Public Sub New()
      mText = New DBText
      mText.TextString = "Temporary entities (Draw)"
      mText.Position = New Point3d(0, 0, 0)
      mCircle = New Circle(New Point3d(0.5, 0, 0), New Vector3d(0, 0, 1), 0.25)
      mArc = New Arc(New Point3d(1, 0, 0), New Vector3d(0, 0, 1), 0.25, Math.PI / 2, -Math.PI / 2)
    End Sub

    'Draw using temporary entities - calling Geometry.Draw.
    Public Overrides Function WorldDraw(ByVal drawable As Autodesk.AutoCAD.GraphicsInterface.Drawable, ByVal wd As Autodesk.AutoCAD.GraphicsInterface.WorldDraw) As Boolean
      ' Calculate matrix to transform from WCS to Line's OCS
      Dim myLine As Line = drawable
      Dim OCS_OriginPt As Point3d = myLine.StartPoint
      Dim OCS_XVec As Vector3d = myLine.Delta.GetNormal
      Dim OCS_ZVec As Vector3d = myLine.Normal
      Dim OCS_YVec As Vector3d = OCS_ZVec.CrossProduct(OCS_XVec)
      ' We don't want text to scale with the line length, so we calculate an 'unscaled' OCS for the text
      Dim unscaledOCS_Mat As Matrix3d = Matrix3d.AlignCoordinateSystem(Point3d.Origin, Vector3d.XAxis, Vector3d.YAxis, Vector3d.ZAxis, OCS_OriginPt, OCS_XVec, OCS_YVec, OCS_ZVec)
      ' We do want the circle and arc to scale with the line, so we calculate a 'full' (corrctly scaled) OCS 
      Dim OCS_Mat As Matrix3d = Matrix3d.Scaling(myLine.Length, myLine.StartPoint) * unscaledOCS_Mat
      ' Push our scaled transform 
      wd.Geometry.PushModelTransform(OCS_Mat)
      ' Draw the arc and circle
      wd.Geometry.Draw(mCircle)
      wd.Geometry.Draw(mArc)
      ' Remove our scaled transform
      wd.Geometry.PopModelTransform()
      ' push our unscaled transform
      wd.Geometry.PushModelTransform(unscaledOCS_Mat)
      ' Draw the text
      wd.Geometry.Draw(mText)
      wd.Geometry.PopModelTransform()
      ' Tell line to draw itself
      Return MyBase.WorldDraw(drawable, wd)
    End Function

    Protected Overrides Sub Dispose(ByVal disposing As Boolean)
      If disposing And Not Me.IsDisposed Then
        If Not mText Is Nothing Then
          mText.Dispose()
          mText = Nothing
        End If
        If Not mCircle Is Nothing Then
          mCircle.Dispose()
          mCircle = Nothing
        End If
        If Not mArc Is Nothing Then
          mArc.Dispose()
          mArc = Nothing
        End If

      End If

      MyBase.Dispose(disposing)
    End Sub


  End Class
#End Region


#Region "Overrule using Draw method"
  Public Class WorldDrawDrawOverrule
    Inherits DrawableOverrule

    ' Draw using temporary entities - calling WorldDraw
    ' Not using Push/PopTransform to demonstrate transforming entity instead of transforming GI coordinate system. I could have used them if I wished.
    Public Overrides Function WorldDraw(ByVal drawable As Autodesk.AutoCAD.GraphicsInterface.Drawable, ByVal wd As Autodesk.AutoCAD.GraphicsInterface.WorldDraw) As Boolean
      ' Calculate matrix to transform from WCS to Line's OCS
      Dim myLine As Line = drawable
      Dim OCS_OriginPt As Point3d = myLine.StartPoint
      Dim OCS_XVec As Vector3d = myLine.Delta.GetNormal
      Dim OCS_ZVec As Vector3d = myLine.Normal
      Dim OCS_YVec As Vector3d = OCS_ZVec.CrossProduct(OCS_XVec)
      ' We don't want text to scale with the line length, so we calculate an 'unscaled' OCS for the text
      Dim unscaledOCS_Mat As Matrix3d = Matrix3d.AlignCoordinateSystem(Point3d.Origin, Vector3d.XAxis, Vector3d.YAxis, Vector3d.ZAxis, OCS_OriginPt, OCS_XVec, OCS_YVec, OCS_ZVec)
      ' We do want the circle and arc to scale with the line, so we calculate a 'full' (corrctly scaled) OCS 
      Dim OCS_Mat As Matrix3d = Matrix3d.Scaling(myLine.Length, myLine.StartPoint) * unscaledOCS_Mat
      ' Draw the arc and circle
      Using circ As Circle = New Circle(New Point3d(0.5, 0, 0), New Vector3d(0, 0, 1), 0.25)
        circ.TransformBy(OCS_Mat)
        circ.WorldDraw(wd)
      End Using
      Using arc As Arc = New Arc(New Point3d(1, 0, 0), New Vector3d(0, 0, 1), 0.25, Math.PI / 2, -Math.PI / 2)
        arc.TransformBy(OCS_Mat)
        arc.WorldDraw(wd)
      End Using
      ' Draw the text
      Using txt As DBText = New DBText
        txt.TextString = "Temporary entities (WorldDraw)"
        txt.Position = New Point3d(0, 0, 0)
        txt.TransformBy(unscaledOCS_Mat)
        txt.WorldDraw(wd)
      End Using
      ' Tell line to draw itself
      Return MyBase.WorldDraw(drawable, wd)
    End Function



  End Class
#End Region



End Namespace