﻿' (C) Copyright 2010 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted,
' provided that the above copyright notice appears in all copies and
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject
' to restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

Namespace DigSigStamp

  Public Class MyCommands

    ' We use this name to store our custom data

    Private Const mDictName As String = "ADNP_DS"

    Friend Shared ReadOnly Property DictName() As String
      Get
        Return mDictName
      End Get
    End Property

    'Remove demand load settings from registry

    <CommandMethod("ADNPLUGINS", "REMOVEDS", _
                CommandFlags.Modal)> _
    Public Shared Sub RemoveDP()

      DemandLoading.RegistryUpdate.UnregisterForDemandLoading()

      Dim doc As Document = _
        Application.DocumentManager.MdiActiveDocument
      Dim ed As Editor = doc.Editor

      ed.WriteMessage(vbCr & _
        "DigSigStamp will not be loaded" _
        & " automatically in future editing sessions.")

    End Sub


    ' Adds Xrecord to the block the user wants to be overruled with
    '  our 'good' and 'bad' blocks.
    ' It's the name of the entry in the extension dictionary that
    '  our  overrule uses to know which blockrefenreces should be
    '  overruled.

    <CommandMethod("ADNPLUGINS", "DSSETSIG", _
                   CommandFlags.Modal)> _
    Public Shared Sub SetAsSignature()

      Dim doc As Document = _
        Application.DocumentManager.MdiActiveDocument
      Dim db As Database = doc.Database
      Dim ed As Editor = doc.Editor

      Dim opts As New PromptEntityOptions( _
        vbLf & "Select Block Reference to overrule: ")
      opts.SetRejectMessage( _
        vbLf & "That isn't a BlockReference!" & vbLf)
      opts.AddAllowedClass(MyPlugin.OverruleClassType, True)
      Dim res As PromptEntityResult = ed.GetEntity(opts)

      If res.Status <> PromptStatus.OK Then Exit Sub

      Using tr As Transaction = _
        db.TransactionManager.StartTransaction()

        Dim br As BlockReference = _
          DirectCast( _
            tr.GetObject(res.ObjectId, OpenMode.ForRead),  _
            BlockReference)

        ' Get ext. dict or create it if it doesn't yet exist

        Dim extDictId As ObjectId = br.ExtensionDictionary
        If Not extDictId.IsValid Then
          br.UpgradeOpen()
          br.CreateExtensionDictionary()
          extDictId = br.ExtensionDictionary
        End If
        Dim extDict As DBDictionary = _
          DirectCast( _
            tr.GetObject(extDictId, OpenMode.ForRead),  _
            DBDictionary)

        ' Add an entry to the extension dictionary

        If extDict.Contains(DictName) Then
          extDict.Remove(DictName)
          ed.WriteMessage( _
            "Signature display override removed from this " & _
            "Block Reference." & vbLf)
        Else
          Using dict As New DBDictionary
            extDict.UpgradeOpen()
            extDict.SetAt(DictName, dict)
            tr.AddNewlyCreatedDBObject(dict, True)
            ed.WriteMessage( _
              "Signature display override set for this " & _
              "Block Reference." & vbLf)
          End Using
          tr.Commit()
        End If

      End Using

      ed.Regen()

    End Sub


    ' This function allows us to select which blocktablerecords to 
    '  draw for our 'signed' and 'unsigned' graphics. This would be
    '  nicer as a dialog, but sticking to Plugin of the Month 
    '  style of keeping everything on command line.
    ' If there's a problem, the top level transaction won't be 
    '  committed and everything is rolled back. But if everything is
    '  successful, then these blocks become anonymous and can't be 
    '  used by the user (e.g. in INSERT function).
    ' We also erase any blockreferences for the newly anonymized
    ' block.

    <CommandMethod("ADNPLUGINS", "DSDEFSIGBLOCKS", _
                   CommandFlags.Modal)> _
    Public Shared Sub setSigBlocks()

      Dim doc As Document = _
        Application.DocumentManager.MdiActiveDocument
      Dim db As Database = doc.Database
      Dim ed As Editor = doc.Editor

      Dim goodId As ObjectId
      Dim badId As ObjectId
      Dim layerId As ObjectId

      Using tr As Transaction = _
        db.TransactionManager.StartTransaction()

        ed.WriteMessage( _
          vbLf & "*** CAUTION: Setting signature blocks will " & _
          "make them unusable for other purposes ***" & vbLf)

        ' If we had old signature blocks in our drawing, we want to
        '  erase the hidden blockrefs to them before we create our 
        '  new ones. Erasing all refs to an anonymous block will
        '  cause it to be purged on save.
        ' (See comments below about why we use hidden blockrefs)

        Dim rb As ResultBuffer = GetNODData(db)
        If Not rb Is Nothing Then
          EraseRefs(DirectCast(rb.AsArray(0).Value, ObjectId))
          EraseRefs(DirectCast(rb.AsArray(1).Value, ObjectId))
          layerId = DirectCast(rb.AsArray(2).Value, ObjectId)
        End If

        ' Prompt for block to represent our 'good' (valid sig.) 

        goodId = PromptForBlockName( _
        "Type name of Block to display for VALID signature (or ?): ")
        If goodId = ObjectId.Null Then
          ed.WriteMessage(vbLf & "Aborting command" & vbLf)
          Exit Sub
        End If

        ' Prompt for block to represent our 'bad' (invalid sig., 
        '  or dirty drawing) graphics

        badId = PromptForBlockName( _
          "Type name of Block to display for INVALID signature " & _
          "(or ?): ")
        If badId = ObjectId.Null Then
          ed.WriteMessage(vbLf & "Aborting command" & vbLf)
          Exit Sub
        End If

        ' Erase any blockrefs for these blocks

        If EraseRefs(goodId) Or EraseRefs(badId) Then
          ed.WriteMessage( _
            "Signature block definitions had Block References. " & _
            "These have been erased." & vbLf)
        End If

        ' Anonymous blocks will be purged on DWG save/reopen if they
        '  have no BlockRefs. We must create blockrefs for them, but
        '  we put them on a hidden layer so user can't see them. 
        ' We also make them invisible for good measure.

        layerId = CreateHiddenBlockRef(goodId, layerId)
        layerId = CreateHiddenBlockRef(badId, layerId)

        ' Store objectids of good block, bad block and hidden layer
        '  in NOD so we can use them later

        rb = New ResultBuffer( _
          New TypedValue(DxfCode.HardPointerId, goodId), _
          New TypedValue(DxfCode.HardPointerId, badId), _
          New TypedValue(DxfCode.HardPointerId, layerId))
        SetNODData(db, rb)

        tr.Commit()

      End Using

      ed.Regen()

    End Sub


    ' Create blockref for block with objectid 'blockId', make it
    '  invisible, and put it on a hidden layer.
    ' layerId is objectid of hidden layer or null if no hidden layer
    '  currently exists.
    ' Function returns objectid of hidden layer.

    Private Shared Function CreateHiddenBlockRef( _
      ByVal blockId As ObjectId, ByVal layerId As ObjectId) _
      As ObjectId

      Dim doc As Document = _
        Application.DocumentManager.MdiActiveDocument
      Dim db As Database = doc.Database

      Using tr As Transaction = _
        db.TransactionManager.StartTransaction()

        ' Create hidden layer if it doesn't exist. Otherwise
        ' retrieve its ObjectId

        Dim lt As LayerTable = _
          DirectCast( _
            tr.GetObject(db.LayerTableId, OpenMode.ForRead),  _
            LayerTable)
        Dim ltr As LayerTableRecord

        ' If layer doesn't exist, then we must create it

        If layerId = ObjectId.Null Or Not lt.Has(layerId) Then
          Using tmpLtr As LayerTableRecord = New LayerTableRecord
            tmpLtr.Name = DictName
            lt.UpgradeOpen()
            layerId = lt.Add(tmpLtr)
            tr.AddNewlyCreatedDBObject(tmpLtr, True)
          End Using
        End If

        ltr = _
          DirectCast( _
            tr.GetObject(layerId, OpenMode.ForWrite),  _
            LayerTableRecord)

        ' Can't assign a hidden layer to an entity, so we must
        '  unhide the layer (if it already existed).

        ltr.IsHidden = False

        ' Create blockref to our anonymous block and put it on our 
        '  hidden layer.

        Using blockRef As _
          New BlockReference(New Point3d(0, 0, 0), blockId)

          ' Put block on hidden layer
          ' Set block as invisible as well as putting it on a hidden
          '  layer

          Dim btr As BlockTableRecord = _
            DirectCast( _
              tr.GetObject( _
                SymbolUtilityServices.GetBlockModelSpaceId(db), _
                OpenMode.ForWrite),  _
              BlockTableRecord)

          btr.AppendEntity(blockRef)
          tr.AddNewlyCreatedDBObject(blockRef, True)
          blockRef.LayerId = layerId
          blockRef.Visible = False
        End Using

        ' Now we hide the layer. This won't show up in the UI or
        '  when iterating the latertablerecord.
        ' That's why we have to store its ObjectId

        ltr.IsHidden = True

        tr.Commit()

      End Using

      Return layerId

    End Function


    ' Erase all blockreferences of blocktablerecord with ObjectId
    '  blockId.
    ' Assumes blockId is valid blocktablerecord objectid.
    ' Returns true if blockreferences were erased.

    Private Shared Function EraseRefs(ByVal blockId As ObjectId) _
      As Boolean

      Dim doc As Document = _
        Application.DocumentManager.MdiActiveDocument
      Dim db As Database = doc.Database

      Dim hasRefs As Boolean

      Using tr As Transaction = _
        db.TransactionManager.StartTransaction()

        Dim btr As BlockTableRecord = _
          DirectCast( _
            tr.GetObject(blockId, OpenMode.ForRead),  _
            BlockTableRecord)

        Dim refs As ObjectIdCollection = _
          btr.GetBlockReferenceIds(True, False)

        For Each refId As ObjectId In refs
          hasRefs = True
          Dim obj As DBObject = _
            tr.GetObject(refId, OpenMode.ForWrite)
          obj.Erase()
        Next

        tr.Commit()

      End Using

      Return hasRefs

    End Function



    ' Ask user to enter name of block to set as signature block
    ' This function validates the name entered to be sure its a
    '   usable blockname, but defers checking that the block
    '   exists to AnonymizeBlock function

    Private Shared Function PromptForBlockName( _
      ByVal promptString As String) As ObjectId

      Dim doc As Document = _
        Application.DocumentManager.MdiActiveDocument
      Dim ed As Editor = doc.Editor

      Dim res As PromptResult
      Dim objId As ObjectId

      ' Ask user for name of block to anonymize

      Do
        Dim opts As New PromptStringOptions(vbLf & promptString)
        res = ed.GetString(opts)
        If res.Status <> PromptStatus.OK Then Exit Function
        If res.StringResult = "?" Then
          ListBlocks()
        End If
      Loop While res.StringResult = "?"

      ' Make named block anonymous

      Try
        SymbolUtilityServices.ValidateSymbolName( _
          res.StringResult, True)
        objId = AnonymizeBlock(res.StringResult)
      Catch ex As Autodesk.AutoCAD.Runtime.Exception
        ed.WriteMessage( _
          "That isn't a valid block name. Aborting command" & vbLf)
      End Try

      Return objId

    End Function


    ' Stores provided resultbuffer in the  named objects dictionary

    Friend Shared Sub SetNODData( _
      ByVal db As Database, ByVal resbuf As ResultBuffer)

      ' Go to NOD, open or create our custom dictionary, open or 
      '  create our Xrecord, set its data.

      Using tr As Transaction = _
        db.TransactionManager.StartTransaction()

        Dim myDict As DBDictionary
        Dim nod As DBDictionary = _
          DirectCast( _
            tr.GetObject(db.NamedObjectsDictionaryId, _
              OpenMode.ForRead),  _
            DBDictionary)

        ' If my dictionary doesn't exist yet, then we add it

        If Not nod.Contains(DictName) Then
          Using dict = New DBDictionary
            nod.UpgradeOpen()
            nod.SetAt(DictName, dict)
            tr.AddNewlyCreatedDBObject(dict, True)
          End Using
        End If

        ' Now we open myDict for write - ready to add new data

        myDict = _
          DirectCast( _
            tr.GetObject(nod.GetAt(DictName), OpenMode.ForWrite),  _
            DBDictionary)

        ' If my xrecord doesn't exist yet, then we add it

        If Not myDict.Contains(DictName) Then
          Using newXrec As New Xrecord
            myDict.SetAt(DictName, newXrec)
            tr.AddNewlyCreatedDBObject(newXrec, True)
          End Using
        End If

        ' Add new data to xrecord

        Dim xrec As Xrecord = _
          DirectCast( _
            tr.GetObject(myDict.GetAt(DictName), _
              OpenMode.ForWrite),  _
            Xrecord)
        xrec.Data = resbuf

        tr.Commit()

      End Using

    End Sub


    ' Retrieve data we stored in the NOD (as a resultbuffer).

    Friend Shared Function GetNODData(ByVal db As Database) _
      As ResultBuffer

      Dim rb As ResultBuffer = Nothing

      ' Likely to be calling this from worlddraw, so we don't want
      '  to use 'real' transactions.
      '  Weird graphical effects if we do.
      Try
      Using tr As Transaction = _
        db.TransactionManager.StartOpenCloseTransaction()

        Dim myDict As DBDictionary
        Dim nod As DBDictionary = _
          DirectCast( _
            tr.GetObject(db.NamedObjectsDictionaryId, _
              OpenMode.ForRead),  _
            DBDictionary)
        If nod.Contains(DictName) Then

          ' Get our dictionary

          myDict = _
            DirectCast( _
              tr.GetObject(nod.GetAt(DictName), OpenMode.ForRead),  _
              DBDictionary)

          ' Now get xrecord xrecord

          If myDict.Contains(DictName) Then
            Dim xrec As Xrecord = _
              DirectCast( _
                tr.GetObject(myDict.GetAt(DictName), _
                  OpenMode.ForRead),  _
                Xrecord)
            rb = xrec.Data
          End If
        End If
        tr.Commit()

      End Using
      Catch ex As Autodesk.AutoCAD.Runtime.Exception

        ' Silently catch AutoCAD exception.
        ' We'll let calling function handle null resbuf

      End Try

      ' We'll either return Nothing or the resbuf from the xrecord

      Return rb

    End Function



    ' Sets blocktablerecord with name blockName to anonymous block.
    ' AutoCAD adds suffix to *U to ensure anonymous name is unique.
    ' Anonymous blocks can't be inserted in DWG by user, but they
    ' are accessible via the API.

    Private Shared Function AnonymizeBlock( _
      ByVal blockName As String) As ObjectId

      Dim doc As Document = _
        Application.DocumentManager.MdiActiveDocument
      Dim db As Database = doc.Database
      Dim ed As Editor = doc.Editor

      Dim btrId As ObjectId
      Using tr As Transaction = _
        db.TransactionManager.StartTransaction()

        Dim btr As BlockTableRecord
        Dim bt As BlockTable = _
          DirectCast( _
            tr.GetObject(db.BlockTableId, OpenMode.ForRead),  _
            BlockTable)
        If bt.Has(blockName) Then
          btrId = bt.Item(blockName)
          btr = _
            DirectCast( _
              tr.GetObject(btrId, OpenMode.ForWrite),  _
              BlockTableRecord)
          btr.Name = "*U"
          ed.WriteMessage( _
            vbLf & "Signature verification block set." & vbLf)
        Else
          ed.WriteMessage( _
            vbLf & "That blockname doesn't exist." & vbLf)
        End If

        tr.Commit()

      End Using

      Return btrId

    End Function


    ' Send list of blocktablerecord names in this drawing to the 
    '  command line.
    ' Called if user types '?' when requested to specify a
    '  signature block name.

    Private Shared Sub ListBlocks()

      Dim doc As Document = _
        Application.DocumentManager.MdiActiveDocument
      Dim db As Database = doc.Database
      Dim ed As Editor = doc.Editor

      Using tr As Transaction = _
        db.TransactionManager.StartTransaction()

        Dim bt As BlockTable = _
          DirectCast( _
            tr.GetObject(db.BlockTableId, OpenMode.ForRead),  _
            BlockTable)
        For Each btrId As ObjectId In bt
          Dim btr As BlockTableRecord = _
            DirectCast( _
              tr.GetObject(btrId, OpenMode.ForRead),  _
              BlockTableRecord)
          If btr.Name.Chars(0) <> "*" Then
            ed.WriteMessage(vbLf & btr.Name)
          End If
        Next
        ed.WriteMessage(vbLf)

        tr.Commit()

      End Using

    End Sub

  End Class

End Namespace