﻿' (C) Copyright 2010 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted,
' provided that the above copyright notice appears in all copies and
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject
' to restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.GraphicsInterface
Imports System.Runtime.InteropServices
Imports System.Collections.Generic
Imports ACSIGNCORE16Lib

Namespace DigSigStamp

  Public Class DSOverruleBag
    Implements IDisposable

    Private Shared mRegisterClass As RXClass = _
      RXClass.GetClass(GetType(BlockReference))
    Private mOverrules As List(Of Overrule)

    'Two versions of New - if no RXClass is passed in then we 
    '  instantiate the overrules, but don't regieter them

    Public Sub New()
      MyOverrules = New List(Of Overrule)
      MyOverrules.Add(New DSDrawOverrule(Me))
    End Sub

    Public Sub New(ByVal regClass As RXClass)
      MyOverrules = New List(Of Overrule)
      MyOverrules.Add(New DSDrawOverrule(Me))
      RegisterClass = regClass
      If Not regClass Is Nothing Then
        Register(RegisterClass)
      End If
    End Sub

    ' Set the class that overrules are registered for

    Friend Shared Property RegisterClass() As RXClass
      Get
        Return mRegisterClass
      End Get
      Set(ByVal value As RXClass)
        mRegisterClass = value
      End Set
    End Property


    ' List of overrules applied by DSOverruleBag

    Private Property MyOverrules() As  _
      System.Collections.Generic.List(Of Overrule)
      Get
        Return mOverrules
      End Get
      Set(ByVal value As List(Of Overrule))
        mOverrules = value
      End Set
    End Property


    ' Register and Unregister will normally be called from this
    '  class's New and Dispose methods. However, they may be called
    '  from another class to register the overrules against more
    '  than one class.

    Friend Sub Register(ByVal regClass As RXClass)
      If Not regClass Is Nothing Then
        For Each orule As Overrule In MyOverrules
          Overrule.AddOverrule(regClass, orule, False)
          orule.SetExtensionDictionaryEntryFilter( _
            MyCommands.DictName)
        Next
        Overrule.Overruling = True
      End If
    End Sub

    Friend Sub Register()
      Register(RegisterClass)
    End Sub

    Friend Sub Unregister(ByVal regClass As RXClass)
      If Not regClass Is Nothing Then
        For Each orule As Overrule In MyOverrules
          Overrule.RemoveOverrule(regClass, orule)
        Next
        Overrule.Overruling = False
      End If
    End Sub

    Friend Sub Unregister()
      Unregister(RegisterClass)
    End Sub


    ' Our one and only overrule

    Private Class DSDrawOverrule
      Inherits DrawableOverrule

      ' DSOverruleBag owns this Overrule. This stores a link back to 
      '  the parent class.

      Private mContainer As DSOverruleBag

      Friend Sub New(ByVal container As DSOverruleBag)
        mContainer = container
      End Sub


      ' Convert AdskTime to DateTime. Time is stored in universal 
      '  time.
      Function AdskTimeToDateTime(ByVal theTime _
                      As ACSIGNCORE16Lib.AdskSystemTime) As String

        If theTime Is Nothing Then
          Return "<Not set>"
        End If
        Dim theDate As New  _
          DateTime(theTime.get_Year, theTime.get_Month, _
          theTime.get_Day, theTime.get_Hour, theTime.get_Minutes, _
          theTime.get_Seconds)
        Return theDate.ToString
      End Function


      Public Overrides Function SetAttributes( _
        ByVal drawable As Drawable, _
        ByVal traits As DrawableTraits) As Integer

        ' Setting '256' bit is setting kDrawableRegenDraw
        '  (Instructs the GS to regenerate this drawable on every
        '  display redraw). Prevents caching of graphics in
        '  3d shademodes.

        Dim ret As Integer = 256 Or _
          MyBase.SetAttributes(drawable, traits)
        Return ret
      End Function


      ' Draw custom graphics for our blockreference

      Public Overrides Function WorldDraw(ByVal drawable As  _
        Drawable, ByVal wd As WorldDraw) As Boolean

        Dim displayBR As BlockReference = Nothing
        Dim flag As Boolean = True
        Dim myBR As BlockReference = _
          DirectCast(drawable, BlockReference)
        Dim goodId As ObjectId
        Dim badId As ObjectId
        Dim db As Database = wd.Context.Database
          If Not db Is Nothing Then
        Dim rb As ResultBuffer = MyCommands.GetNODData(db)
        If Not rb Is Nothing Then
          goodId = DirectCast(rb.AsArray(0).Value, ObjectId)
          badId = DirectCast(rb.AsArray(1).Value, ObjectId)
        End If
          End If

        ' If we don't have our signature block objectids stored,
        '  then don't mess about with transactions - just ask block
        '  to draw itself and exit function.

        If goodId = ObjectId.Null Or badId = ObjectId.Null Then
          Return MyBase.WorldDraw(drawable, wd)
        End If
        Try
          If Not db Is Nothing Then
            Using tr As Transaction = _
              db.TransactionManager.StartOpenCloseTransaction()

              ' Get digital signature info (if it exists)

              Dim si As SigInfo = _
                GetSigInfo(db, wd.Context.IsPlotGeneration)
              If si Is Nothing Then

                ' Drawing does not have valid signature or is dirty 
                '  (DBMOD > 0).(See comments about special case of 
                ' plottin/publishing in GetSigInfo function).

                displayBR = _
                  New BlockReference(New Point3d(0, 0, 0), badId)
              Else

                ' Drawing has valid digital sig. and is not dirty.

                displayBR = _
                  New BlockReference(New Point3d(0, 0, 0), goodId)

                ' Note we assume 'bad' block does not have
                '  attributes, so we don't attempt to draw them.

                Dim goodBTR As BlockTableRecord = _
                  DirectCast( _
                    tr.GetObject(goodId, OpenMode.ForRead),  _
                    BlockTableRecord)

                ' Need to draw any attributes too

                If goodBTR.HasAttributeDefinitions Then
                  For Each attId As ObjectId In goodBTR
                    If attId.ObjectClass = _
                      RXClass.GetClass( _
                        GetType(AttributeDefinition)) Then

                      Dim attDef As AttributeDefinition
                      attDef = _
                        DirectCast( _
                          tr.GetObject(attId, OpenMode.ForRead),  _
                          AttributeDefinition)

                      Using attRef As AttributeReference = _
                          New AttributeReference
                        attRef.SetDatabaseDefaults()
                        attRef.SetAttributeFromBlock( _
                          attDef, myBR.BlockTransform)
                        Dim bFlag As Boolean

                        ' Attribute definitions in 'good' 
                        '  blocktablerecord need to follow naming 
                        '  convention.

                        Select Case attDef.Tag
                          Case "CERTISSUER"
                            attRef.TextString = si.certIssuer

                          Case "CERTEMAIL"
                            attRef.TextString = si.certEmail

                          Case "CERTSN"
                            attRef.TextString = si.certSN

                          Case "CERTSUBJECT"
                            attRef.TextString = si.certX500

                          Case "CERTCOMMENT"
                            attRef.TextString = si.certComment

                          Case "CERTNOTBEFORE"
                            attRef.TextString = _
                          AdskTimeToDateTime(si.certNotBefore)

                          Case "CERTEXPIRES"
                            attRef.TextString = _
                            AdskTimeToDateTime(si.certExpires)

                          Case "CERTTIMESERVER"
                            attRef.TextString = _
                              si.certTimeServer

                          Case "CERTTIMESTAMP"
                            attRef.TextString = _
                          AdskTimeToDateTime(si.certTimeStamp)
                          Case Else
                            bFlag = True
                        End Select

                        If Not bFlag And Not attRef Is Nothing Then
                          attRef.WorldDraw(wd)
                        End If
                      End Using
                    End If
                  Next
                End If
              End If
              tr.Commit()
            End Using
          End If

        Catch ex As Autodesk.AutoCAD.Runtime.Exception

          ' We'll assume if an AutoCAD exception was thrown that the
          '  error is ignorable, so we'll just allow the code to draw
          '  the normal blockreference.

        Finally

          ' Draw our overrule graphics

          If Not displayBR Is Nothing Then
            displayBR.SetDatabaseDefaults()
            displayBR.BlockTransform = myBR.BlockTransform
            displayBR.WorldDraw(wd)
            displayBR.Dispose()
          Else
            flag = MyBase.WorldDraw(drawable, wd)
          End If
        End Try
        Return flag
      End Function

      ' Retrieve digital signature info.
      ' Returns nothing if drawing is unsigned, signature not valid,
      '  or DBMOD > 0.

      Private Function GetSigInfo( _
        ByVal db As Database, ByVal isPlotting As Boolean) As SigInfo

        ' If DBMOD > 0 then drawing has been modified, so digital 
        '  signature is not valid regardless of other tests. However,
        '  plotting will make DBMOD non-zero so we need to trap that 
        '  special case. We do this by adding userdata to the
        '  document to flag whether DBMOD was zero 'just before'
        '  plotting.
        ' This is taken care of by the document events setup by 
        '  myPlugin class - DBModProxy is checking for that.

        Dim DBMOD As Short = _
          DirectCast( _
            Application.GetSystemVariable("DBMOD"), _
            Short)
        If DBMOD <> 0 And DBMODProxy(db) Then
          Return Nothing
        End If

        ' Now attempt to get digital signature info
        ' Note ACSIGNCORE16Lib is not part of the 'official' AutoCAD
        '  API so it may change without warning in a future release.
        ' There is no 'official' AutoCAD API that works as we want it
        '  to.

        If db.Filename <> "" Then
          Dim res As Integer
          Dim sgn As New ACSIGNCORE16Lib.AdskSignDWG
          sgn.IsFileSigned(db.Filename, Nothing, res)
          If res <> 0 Then
            Dim res2 As Integer = 0
            Try

              ' If drawing is signed, we also need to check that 
              '  signature is valid.
              ' A COMException is thrown if signature isn't valid

              sgn.VerifySign(db.Filename, Nothing, res2)
            Catch ex As COMException
              res2 = 0
            End Try
            If res2 <> 0 Then

              ' This is the only place we return true. 

              Dim si As New SigInfo
              sgn.ExtractSignatureInfo(db.Filename, Nothing, _
                si.certSubject, si.certIssuer, _
                si.certEmail, si.certSN, _
                si.certX500, si.certComment, _
                si.certNotBefore, _
                si.certExpires, _
                si.certTimeServer, _
                si.certTimeStamp)
              si.Validate()
              Return si
            End If
          End If
        End If

        'If we reach here, then we didn't have a valid signature.

        Return Nothing
      End Function


      ' Function checks userdata to see if document digital signature
      '  was valid before the last call. See comments and event 
      '  handlers in myPlugin class (myPlugin.vb)

      Private Function DBMODProxy(ByVal db As Database) As Boolean
        Dim doc As Document = _
          Application.DocumentManager.GetDocument(db)
        If doc.UserData.Contains(MyCommands.DictName) Then
          Return False
        Else
          Return True
        End If
      End Function


      ' Used to store Digital Signature info
      ' Attribute tags attached to the 'good' block should be the
      '  same as the variable names in this class

      Private Class SigInfo
        Public certSubject As String
        Public certIssuer As String
        Public certEmail As String
        Public certSN As String
        Public certX500 As String
        Public certComment As String
        Public certTimeServer As String
        Public certNotBefore As ACSIGNCORE16Lib.AdskSystemTime
        Public certExpires As ACSIGNCORE16Lib.AdskSystemTime
        Public certTimeStamp As ACSIGNCORE16Lib.AdskSystemTime

        ' Ensures strings are assigned a value, even if they were set
        '  to nothing

        Public Sub Validate()
          If certSubject Is Nothing Then
            certSubject = "<Not set>"
          End If
          If certIssuer Is Nothing Then
            certIssuer = "<Not set>"
          End If
          If certEmail Is Nothing Then
            certEmail = "<Not set>"
          End If
          If certSN Is Nothing Then
            certSN = "<Not set>"
          End If
          If certX500 Is Nothing Then
            certX500 = "<Not set>"
          End If
          If certComment Is Nothing Then
            certComment = "<Not set>"
          End If
          If certTimeServer Is Nothing Then
            certTimeServer = "<Not set>"
          End If
        End Sub
      End Class
    End Class

    Private disposedValue As Boolean = False

    ' IDisposable suppose

    Protected Overridable Overloads Sub Dispose( _
      ByVal disposing As Boolean)

      If Not Me.disposedValue Then
        If disposing Then
          Unregister(RegisterClass)
          For Each orule As Overrule In MyOverrules
            orule.Dispose()
          Next
          mOverrules = Nothing
        End If
      End If
      Me.disposedValue = True
    End Sub


#Region " IDisposable Support "

    ' This code added by Visual Basic to correctly implement the 
    '  disposable pattern.

    Public Overloads Sub Dispose() Implements IDisposable.Dispose

      ' Do not change this code.  Put cleanup code in Dispose(ByVal
      '  disposing As Boolean) above.

      Dispose(True)
      GC.SuppressFinalize(Me)
    End Sub

    Protected Overrides Sub Finalize()
      Dispose(False)
      MyBase.Finalize()
    End Sub

#End Region

  End Class

End Namespace